@@ -1,8 +1,11 @@
+from collections import defaultdict
 from copy import copy
+from itertools import chain
 from typing import Any, Union, cast
 
 from graphql import (
     GraphQLField,
+    GraphQLInterfaceType,
     GraphQLList,
     GraphQLNonNull,
     GraphQLObjectType,
@@ -23,9 +26,32 @@ from ..printer import print_schema
 from ..schema import Schema as BaseSchema
 
 
+def _find_directives(schema):
+    all_graphql_types = schema._schema.type_map.values()
+
+    directives = []
+
+    for type_ in all_graphql_types:
+        strawberry_definition = type_.extensions.get("strawberry-definition")
+
+        if not strawberry_definition:
+            continue
+
+        directives.extend(strawberry_definition.directives)
+
+        fields = getattr(strawberry_definition, "fields", [])
+        values = getattr(strawberry_definition, "values", [])
+
+        for field in chain(fields, values):
+            directives.extend(field.directives)
+
+    return directives
+
+
 class Schema(BaseSchema):
     def __init__(self, *args, **kwargs):
         additional_types = list(kwargs.pop("types", []))
+        enable_federation_2 = kwargs.pop("enable_federation_2", False)
 
         kwargs["types"] = additional_types
 
@@ -35,14 +61,17 @@ class Schema(BaseSchema):
         self._create_service_field()
         self._extend_query_type()
 
+        if enable_federation_2:
+            self._add_link_directives()
+
     def entities_resolver(self, root, info, representations):
         results = []
 
         for representation in representations:
             type_name = representation.pop("__typename")
-            type = self.schema_converter.type_map[type_name]
+            type_ = self.schema_converter.type_map[type_name]
 
-            definition = cast(TypeDefinition, type.definition)
+            definition = cast(TypeDefinition, type_.definition)
             resolve_reference = definition.origin.resolve_reference
 
             func_args = get_func_args(resolve_reference)
@@ -60,6 +89,29 @@ class Schema(BaseSchema):
 
         self._schema.type_map["_Any"] = self.Any
 
+    def _add_link_directives(self):
+        from .schema_directives import FederationDirective, Link
+
+        all_directives = _find_directives(self)
+
+        directive_by_url = defaultdict(set)
+
+        for directive in all_directives:
+            if isinstance(directive, FederationDirective):
+                directive_by_url[directive.imported_from.url].add(
+                    f"@{directive.imported_from.name}"
+                )
+
+        link_directives = tuple(
+            Link(
+                url=url,
+                import_=list(sorted(directives)),
+            )
+            for url, directives in directive_by_url.items()
+        )
+
+        self.schema_directives = tuple(self.schema_directives) + link_directives
+
     def _extend_query_type(self):
         fields = {"_service": self._service_field}
 
@@ -111,6 +163,8 @@ def _get_entity_type(type_map: TypeMap):
         type.implementation
         for type in type_map.values()
         if _has_federation_keys(type.definition)
+        # TODO: check this
+        and not isinstance(type.implementation, GraphQLInterfaceType)
     ]
 
     # If no types are annotated with the key directive, then the _Entity
